/******************************************************************************
   App.c
   The application.
******************************************************************************/
#include <MC9S12E128.h>		// Derivative information
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "Datatypes.h"
#include "App.h"
#include "Timer.h"
#include "KBD.h"
#include "LCD.h"
#include "ATD.h"
#include "PWM.h"
#include "SCI0.h"


// Global Variables
extern BOOL InputVoltageGood;
extern BOOL BusVoltageGood;
extern BOOL AC_VoltagePresent;
extern BOOL ErrorDetected;




// Variables used by thread 1
static tPointerFcn Th1FuncPtr = Th1Func1;	// Starting state
UINT8 DelayTimer;
UINT8 SwData;

// Variables used by thread 2
static tPointerFcn Th2FuncPtr = Th2Func1;


// Variables used by thread 3
static tPointerFcn Th3FuncPtr = Th3Func1;
UINT8 LED_Timer;		// Timer handle for LED blinling
UINT8 LED_State = 0;	// The state of LED lights in the push-buttons


/*****************************************************************************/
// THREAD 1
/*****************************************************************************/
/******************************************************************************
   Thread 1 State Machine */

// Call this function in main loop
void GoToThread1(void)
{
	Th1FuncPtr();	// Use the function pointer to go to a state
}
/*****************************************************************************/

/* This state machine monitors the push buttons and executes inverter functions
   according to the following sequence:


*/
//-----------------------------------------------------------------------------
// Thread 1 State 1:
void Th1Func1(void)
{
	StartATD();
	DelayTimer = get_timer();
	Th1FuncPtr = Th1Func2;
	SCI0_PutQueue("State 1: Go to State 2 - STANDBY\r\n");
}


//-----------------------------------------------------------------------------
// Thread 1 State 2:	STANDBY (Idle LED blinking)
void Th1Func2(void)
{
	if (InputVoltageGood == FALSE)
	{
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Standby   ");
		LED_State = IdleBlinking;
		Th1FuncPtr = Th1Func3;
		SCI0_PutQueue("State 2: Input voltage is low, go to state 3 - STANDBY\r\n");
	} else
	{
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Stopped   ");
		LED_State = IdleOn;
		Th1FuncPtr = Th1Func4;
		SCI0_PutQueue("State 2: Input voltage is good, go to state 4\r\n");
	}
}


//-----------------------------------------------------------------------------
// Thread 1 State 3:	STANDBY (discard any push button presses)
void Th1Func3(void)
{
	if (ErrorDetected == TRUE)
	{
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Halted    ");
		LED_State = ErrorOn;
		Th1FuncPtr = Th1Func7;
		SCI0_PutQueue("State 3: Error detected, go to state 7\r\n");
		return;
	}

	if (InputVoltageGood == TRUE)
	{
		Th1FuncPtr = Th1Func2;
		SCI0_PutQueue("State 3: Input Voltage is good. Go to State 2\r\n");
		return;
	}

	// InputVoltageGood = FALSE, stay in this state
	// Ignore any switch presses
	SwData = SW_FIFO_ReadData();
	if (SwData > 0)
	{
		switch (SwData)
		{
			case StopKey:
				break;
			case StartKey:
				break;
			case ResetKey:
				break;
		}
	}
}


//-----------------------------------------------------------------------------
// Thread 1 State 4:	STOPPED (Idle LED on)
void Th1Func4(void)
{
	if (ErrorDetected == TRUE)
	{
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Halted    ");
		LED_State = ErrorOn;
		Th1FuncPtr = Th1Func7;
		SCI0_PutQueue("State 4: Error detected, go to state 7\r\n");
		return;
	}

	if (InputVoltageGood == FALSE)
	{
		Th1FuncPtr = Th1Func2;
		SCI0_PutQueue("State 4: Input voltage is low, go to State 2\r\n");
		return;
	}

	SwData = SW_FIFO_ReadData();
	if (SwData > 0)
	{
		switch (SwData)
		{
			case StopKey:
				break;
			case StartKey:
				StartBoostPWM();		// Start boost PWM
				SendCommandBusy(SetLine2);
				SendMsgAtCursor("Mode: Starting  ");
				LED_State = RunBlinking;
				Th1FuncPtr = Th1Func5;	// Go to STARTING
				break;
			case ResetKey:
				break;
		}
	}
}


//-----------------------------------------------------------------------------
// Thread 1 State 5:	STARTING (Loop until bus voltage is good)
void Th1Func5(void)
{
	if (ErrorDetected == TRUE)
	{
		StopBoostPWM();					// Stop boost PWM
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Halted    ");
		LED_State = ErrorOn;
		Th1FuncPtr = Th1Func7;
		SCI0_PutQueue("State 5: Error detected, go to state 7\r\n");
		return;
	}

	if (InputVoltageGood == FALSE)
	{
		StopBoostPWM();					// Stop boost PWM
		Th1FuncPtr = Th1Func2;
		SCI0_PutQueue("State 5: Input voltage is low, go to state 2\r\n");
		return;
	}
	if (BusVoltageGood == TRUE)		// Try this initially
		 //if (AC_VoltagePresent & BusVoltageGood)
	{
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Running   ");
		LED_State = RunOn;
		Th1FuncPtr = Th1Func6;
		SCI0_PutQueue("State 5: Bus voltage is good, go to state 6\r\n");
		return;
	}

	SwData = SW_FIFO_ReadData();
	if (SwData > 0)
	{
		switch (SwData)
		{
			case StopKey:
				StopBoostPWM();			// Stop boost PWM
				SendCommandBusy(SetLine2);
				SendMsgAtCursor("Mode: Stopped   ");
				LED_State = IdleOn;
				Th1FuncPtr = Th1Func4;	// Go to STOPPED
				break;
			case StartKey:
				break;
			case ResetKey:
				break;
		}
	}
}


//-----------------------------------------------------------------------------
// Thread 1 State 6:	RUNNING (Hard stop if an error occurs)
void Th1Func6(void)
{
	if (ErrorDetected == TRUE)
	{
		StopBoostPWM();					// Stop boost PWM
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Halted    ");
		LED_State = ErrorOn;
		Th1FuncPtr = Th1Func7;
		SCI0_PutQueue("State 6: Error detected, go to state 7\r\n");
		return;
	}

	if (InputVoltageGood == FALSE)
	{
		StopBoostPWM();					// Stop boost PWM
		Th1FuncPtr = Th1Func2;
		SCI0_PutQueue("State 6: Input voltage is low, go to state 2\r\n");
		return;
	}

	if (BusVoltageGood == FALSE)
	{
		StartBoostPWM();		// Start boost PWM
		SendCommandBusy(SetLine2);
		SendMsgAtCursor("Mode: Starting  ");
		LED_State = RunBlinking;
		Th1FuncPtr = Th1Func5;	// Go to STARTING
		SCI0_PutQueue("State 6: Bus voltage is low, go to state 5\r\n");
		return;
	}

	SwData = SW_FIFO_ReadData();
	if (SwData > 0)
	{
		switch (SwData)
		{
			case StopKey:
				StopBoostPWM();			// Stop boost PWM
				SendCommandBusy(SetLine2);
				SendMsgAtCursor("Mode: Stopped   ");
				LED_State = IdleOn;
				Th1FuncPtr = Th1Func4;	// Go to STOPPED
				break;
			case StartKey:
				break;
			case ResetKey:
				break;
		}
	}
}


//-----------------------------------------------------------------------------
// Thread 1 State 7:	HALTED (Wait for Reset push button)
void Th1Func7(void)
{
    SwData = SW_FIFO_ReadData();
	if (SwData > 0)
	{
		switch (SwData)
		{
			case StopKey:
				break;
			case StartKey:
				break;
			case ResetKey:
				Th1FuncPtr = Th1Func2;	// Go to STANDBY
				break;
		}
	}
}


//-----------------------------------------------------------------------------
// Thread 1 State 8:	DIAGNOSTIC (Turn on PWM for scoping)
void Th1Func8(void)
{
	StartBoostPWM();		// Start boost PWM
	SendCommandBusy(SetLine2);
	SendMsgAtCursor("Mode: Diagnostic");
	LED_State = RunOn;
	Th1FuncPtr = Th1Func9;	// Go to next state
}


//-----------------------------------------------------------------------------
// Thread 1 State 9:	DIAGNOSTIC (Wait for Stop push button)
void Th1Func9(void)
{
	SwData = SW_FIFO_ReadData();
	if (SwData > 0)
	{
		switch (SwData)
		{
			case StopKey:
				StopBoostPWM();			// Stop boost PWM
				SendCommandBusy(SetLine2);
				SendMsgAtCursor("Mode: Stopped   ");
				LED_State = IdleOn;
				Th1FuncPtr = Th1Func4;	// Go to STOPPED
				break;
			case StartKey:
				break;
			case ResetKey:
				break;
		}
	}
}

//-----------------------------------------------------------------------------
// Thread 1 State 10:
void Th1Func10(void)
{

}

//-----------------------------------------------------------------------------
// Thread 1 State 11:
void Th1Func11(void)
{

}

//-----------------------------------------------------------------------------
// Thread 1 State 12:
void Th1Func12(void)
{

}

//-----------------------------------------------------------------------------
// Thread 1 State 13:
void Th1Func13(void)
{

}


/*****************************************************************************/
// THREAD 2
/*****************************************************************************/
/******************************************************************************
   Thread 2 State Machine */

// Call this function in main loop
void GoToThread2(void)
{
	Th2FuncPtr();
}
/*****************************************************************************/



//-----------------------------------------------------------------------------
// Thread 2 State 1:
void Th2Func1(void)
{
	Th2FuncPtr = Th2Func2;	// Go to State 2
}

//-----------------------------------------------------------------------------
// Thread 2 State 2:
void Th2Func2(void)
{
	Th2FuncPtr = Th2Func3;	// Go to State 3
}

//-----------------------------------------------------------------------------
// Thread 2 State 3:
void Th2Func3(void)
{
	Th2FuncPtr = Th2Func4;	// Go to State 4
}

//-----------------------------------------------------------------------------
// Thread 2 State 4:
void Th2Func4(void)
{
	Th2FuncPtr = Th2Func1;	// Go to State 1
}



/*****************************************************************************/
// THREAD 3
/*****************************************************************************/
/******************************************************************************
   Thread 3 State Machine */

// Call this function in main loop
void GoToThread3(void)
{
	Th3FuncPtr();
}
/*****************************************************************************/

/* This state machine manages the three LEDs in the push buttons. It uses the
   variable LED_State to determine what LED should be lit according to the
   following codes:
   LED_State
      0       All LEDs off
      1       Idle LED on
      2       Run LED on
      3       Error LED on
      4       Idle LED blinking
      6       Run LED blinking
*/
//-----------------------------------------------------------------------------
// Thread 3 State 1:	// This state allocates a timer for blinking LEDs
void Th3Func1(void)
{
	LED_Timer = get_timer();	// Get a timer for blinking
	Th3FuncPtr = Th3Func2;		// Go to State 2
}

//-----------------------------------------------------------------------------
// Thread 3 State 2:	// This state controlls the LEDs according to LED_State
void Th3Func2(void)		// This state, once entered, is executed repeatedly
{
	switch (LED_State)
	{
		case (AllOff):				// All LEDs off
			ClearIdleLED();
			ClearRunLED();
			ClearErrorLED();
			break;

		case (IdleOn):				// Idle LED on
			SetIdleLED();
			ClearRunLED();
			ClearErrorLED();
			break;

		case (RunOn):				// Run LED on
			ClearIdleLED();
			SetRunLED();
			ClearErrorLED();
			break;

		case (ErrorOn):
			ClearIdleLED();			// Error LED on
			ClearRunLED();
			SetErrorLED();
			break;

		case (IdleBlinking):		// Idle LED on for 500 milliseconds
			if (check_timer(LED_Timer)==0)
			{
				SetIdleLED();
				ClearRunLED();
				ClearErrorLED();
				init_timer(LED_Timer,500);
				LED_State = 5;
			}
			break;

		case (5):					// Idle LED off for 500 milliseconds
			if (check_timer(LED_Timer)==0)
			{
				ClearIdleLED();
				ClearRunLED();
				ClearErrorLED();
				init_timer(LED_Timer,500);
				LED_State = 4;
			}
			break;

		case (RunBlinking):			// Run LED on for 500 milliseconds
			if (check_timer(LED_Timer)==0)
			{
				ClearIdleLED();
				SetRunLED();
				ClearErrorLED();
				init_timer(LED_Timer,500);
				LED_State = 7;
			}
			break;

		case (7):					// Run LED off for 500 milliseconds
			if (check_timer(LED_Timer)==0)
			{
				ClearIdleLED();
				ClearRunLED();
				ClearErrorLED();
				init_timer(LED_Timer,500);
				LED_State = 6;
			}
			break;
	}
//		Th3FuncPtr = Th3Func3;	// Go to State 3
}

//-----------------------------------------------------------------------------
// Thread 3 State 3:
void Th3Func3(void)
{
	Th3FuncPtr = Th3Func1;	// Go to State 1
}


/******************************************************************************
   Diagnostic */
void SetDiagMode(void)
{
	Th1FuncPtr = Th1Func8;
}






