/******************************************************************************
   PMF.c

******************************************************************************/

// Include Files
#include <MC9S12E128.h>		// Derivative information
#include <stdio.h>

#include "PMF.h"
#include "ATD.h"
#include "Buffers.h"
#include "SCI0.h"
#include "Datatypes.h"
#include "IO.h"


// Constants
// One-quarter of a sine wave (90 degrees). Only first 126 values are used.
const byte Sine[128] =
{
  0,  3,  5,  8, 10, 13, 15, 18, 20, 23, 25, 28, 30, 33, 35, 37,
 40, 42, 45, 47, 50, 52, 55, 57, 59, 62, 64, 67, 69, 71, 74, 76,
 78, 81, 83, 85, 87, 90, 92, 94, 96, 99,101,103,105,107,109,111,
113,116,118,120,122,124,126,127,129,131,133,135,137,139,141,142,
144,146,148,149,151,152,154,156,157,159,160,162,163,165,166,168,
169,170,172,173,174,175,176,178,179,180,181,182,183,184,185,186,
187,188,189,189,190,191,192,192,193,194,194,195,195,196,196,197,
197,198,198,198,199,199,199,199,200,200,200,200,200,200,200,200
};


// Global Variables
BOOL     Phase;		// Positive and negative half-cycle (2 per cycle)
BOOL     Slope;		// Positive and negative slope (2 positive and 2 negative per cycle)
word     Angle;		// Phase angle of a sign wave
byte     Amp;		// Amplitude of sine wave generated
int      Adj;		// Correction for zero crossing

word     SineValue;	// Value of the sine wave from table


/*****************************************************************************/
#pragma CODE_SEG NON_BANKED
interrupt void PMFA_Reload_Int(void)
{
	if (Phase)		// Positive half-cycle
	{
		if (Slope)	// Positive slope
		{
			SineValue = Sine[Angle++];	// Get sine wave value from table
			PMFVAL0 = (Modulus/2)+(SineValue*Amp/255);	// Calculate PWM 
			if (Angle == 125)
			{
				switch(Adj)
				{
					case +1:		// Add an extra angle cycle
						Angle--;	// Will do 124 over again
						Adj = 0;
						break;

					case 0:			// In sync, no need to adjust
						Slope = 0;	// Change to negative slope
						break;

					case -1:		// Subtract one angle cycle
						Slope = 0;	// Change to negative slope
						Angle--;	// Will not do 125
						Adj = 0;
						break;
				}
			}
		}
		else	// Negative slope
		{
			SineValue = Sine[Angle--];	// Get sine wave value from table
			PMFVAL0 = (Modulus/2)+(SineValue*Amp/255);	// Calculate PWM
			if (Angle == 0)
				Phase = 0;	// Change to negative half-cycle
		}
	}
	else	// Negative half-cycle
	{
		if (!Slope)	// Negative slope
		{
			SineValue = Sine[Angle++];	// Get sine wave value from table
			PMFVAL0 = (Modulus/2)-(SineValue*Amp/255);	// Calculate PWM
			if (Angle == 125)
			{
				switch(Adj)
				{
					case +1:		// Add an extra angle cycle
						Angle--;	// Will do 124 over again
						Adj = 0;
						break;

					case 0:			// In sync, no need to adjust
						Slope = 1;	// Change to positive slope
						break;

					case -1:		// Subtract one angle cycle
						Slope = 1;	// Change to positive slope
						Angle--;	// Will not do 125
						Adj = 0;
						break;
				}
			}
		}
		else	// Positive slope
		{
			SineValue = Sine[Angle--];	// Get sine wave value from table
			PMFVAL0 = (Modulus/2)-(SineValue*Amp/255);	// Calculate PWM
			if (Angle == 0)
				Phase = 1;	// Change to positive half-cycle
		}
	}
	PMFENCA_LDOKA = 1;		// Allow PWM register load
	PMFFQCA_PWMRFA = 1;		// Clear PWM Reload Flag A
}
#pragma CODE_SEG DEFAULT


/*****************************************************************************/
#pragma CODE_SEG NON_BANKED
interrupt void PMFB_Reload_Int(void)
{
	//PMFENCB_LDOKB = 1;		// Allow PWM register load
	PMFFQCB_PWMRFB = 1;		// Clear PWM Reload Flag B
	Toggle_LED();
}
#pragma CODE_SEG DEFAULT


/*****************************************************************************/
#pragma CODE_SEG NON_BANKED
interrupt void PMFC_Reload_Int(void)
{
	//PMFENCC_LDOKC = 1;		// Allow PWM register load
	PMFFQCC_PWMRFC = 1;		// Clear PWM Reload Flag C
	Toggle_LED();
}
#pragma CODE_SEG DEFAULT



/*****************************************************************************/
void InitPMF(void)
{
/*  */
	PMFCFG0 =	(0
				/*| PMFCFG0_INDEPA_MASK*/	// PWM0 and PWM1 are complementart pair	
				/*| PMFCFG0_INDEPB_MASK*/	// PWM2 and PWM3 are complementary pair
				/*| PMFCFG0_INDEPC_MASK*/	// PWM4 and PWM5 are complemantart pair
				| PMFCFG0_EDGEA_MASK		// PWM0 and PWM1 are edge-aligned
				| PMFCFG0_EDGEB_MASK		// PWM2 and PWM3 are edge-aligned
				| PMFCFG0_EDGEC_MASK		// PWM4 and PWM5 are edge-aligned
				| PMFCFG0_MTG_MASK			// Multiple timebase generator
				/*| PMFCFG0_WP_MASK*/		// Write-protected regs can be written
				);

	PMFCFG1	=	(0
				/*| PMFCFG1_TOPNEGA_MASK*/		// Positive PWM0 polarity
				/*| PMFCFG1_BOTNEGA_MASK*/		// Positive PWM1 polarity
				/*| PMFCFG1_TOPNEGB_MASK*/		// Positive PWM2 polarity
				/*| PMFCFG1_BOTNEGB_MASK*/		// Positive PWM3 polarity
				/*| PMFCFG1_TOPNEGC_MASK*/		// Positive PWM4 polarity
				/*| PMFCFG1_BOTNEGC_MASK*/		// Positive PWM5 polarity
				/*| PMFCFG1_ENHA_MASK*/			// Disable hardware acceleration
				);

	PMFCFG2	=	(0
				/*| PMFCFG2_MSK0_MASK*/			// PWM0 is unmasked
				/*| PMFCFG2_MSK1_MASK*/			// PWM1 is unmasked
				/*| PMFCFG2_MSK2_MASK*/			// PWM2 is unmasked
				/*| PMFCFG2_MSK3_MASK*/			// PWM3 is unmasked
				/*| PMFCFG2_MSK4_MASK*/			// PWM4 is unmasked
				/*| PMFCFG2_MSK5_MASK*/			// PWM5 is unmasked
				); 

	PMFCFG3	=	(0
				/*| PMFCFG3_SWAPA_MASK*/		// No swap of PWM0 and PWM1
				/*| PMFCFG3_SWAPB_MASK*/		// No swap pf PWM2 and PWM3
				/*| PMFCFG3_SWAPC_MASK*/		// No swap of PWM4 and PWM5
				/*| PMFCFG3_VLMODE0_MASK*/		// Each value reg is accessed independently
				/*| PMFCFG3_VLMODE1_MASK*/
				/*| PMFCFG3_PMFFRZ_MASK*/		// PMF continues to run in freeze mode
				/*| PMFCFG3_PMFWAI_MASK*/		// PMF continues to run in wait mode
				); 

	PMFFCTL	=	(0
				/*| PMFFCTL_FIE0_MASK*/			// Fault 0 interrupts disabled
				/*| PMFFCTL_FMODE0_MASK*/		// Manual fault clearing of fault 0
				/*| PMFFCTL_FIE1_MASK*/			// Fault 1 interrupts disabled
				/*| PMFFCTL_FMODE1_MASK*/		// Manual fault clearing of fault 1
				/*| PMFFCTL_FIE2_MASK*/			// Fault 2 interrupts disabled
				/*| PMFFCTL_FMODE2_MASK*/		// Manual fault clearing of fault 2
				/*| PMFFCTL_FIE3_MASK*/			// Fault 3 interrupts disabled
				/*| PMFFCTL_FMODE3_MASK*/		// Manual fault clearing of fault 3
				);

	PMFFPIN	=	(0
				/*| PMFFPIN_FPINE0_MASK*/		// Fault 0 pin disabled
				/*| PMFFPIN_FPINE1_MASK*/		// Fault 1 pin disabled
				/*| PMFFPIN_FPINE2_MASK*/		// Fault 2 pin disabled
				/*| PMFFPIN_FPINE3_MASK*/		// Fault 3 pin disabled
				);

	PMFFSTA	=	(0
				| PMFFSTA_FFLAG0_MASK			// Clear all interrupt flags
				| PMFFSTA_FFLAG1_MASK
				| PMFFSTA_FFLAG2_MASK
				| PMFFSTA_FFLAG3_MASK
				);

	PMFQSMP	=	(0
				/*| PMFQSMP_QSMP00_MASK*/			// 1 sample
				/*| PMFQSMP_QSMP01_MASK*/
				/*| PMFQSMP_QSMP10_MASK*/			// 1 sample
				/*| PMFQSMP_QSMP11_MASK*/
				/*| PMFQSMP_QSMP20_MASK*/			// 1 sample
				/*| PMFQSMP_QSMP21_MASK*/
				/*| PMFQSMP_QSMP30_MASK*/			// 1 sample
				/*| PMFQSMP_QSMP31_MASK*/
				);

// PMF Disable Mapping A Register (PMFDMPA)
	PMFDMPA	=	(0
				/*| PMFDMPA_DMP00_MASK*/		// Disable PWM0 fault 0
				/*| PMFDMPA_DMP01_MASK*/		// Disable PWM0 fault 1
				/*| PMFDMPA_DMP02_MASK*/		// Disable PWM0 fault 2
				/*| PMFDMPA_DMP03_MASK*/		// Disable PWM0 fault 3
				/*| PMFDMPA_DMP10_MASK*/		// Disable PWM1 fault 0
				/*| PMFDMPA_DMP11_MASK*/		// Disable PWM1 fault 1
				/*| PMFDMPA_DMP12_MASK*/		// Disable PWM1 fault 2
				/*| PMFDMPA_DMP13_MASK*/		// Disable PWM1 fault 3
				);

// PMF Disable Mapping B Register (PMFDMPB)
	PMFDMPB	=	(0
				/*| PMFDMPB_DMP20_MASK*/		// Disable PWM2 fault 0
				/*| PMFDMPB_DMP21_MASK*/		// Disable PWM2 fault 1
				/*| PMFDMPB_DMP22_MASK*/		// Disable PWM2 fault 2
				/*| PMFDMPB_DMP23_MASK*/		// Disable PWM2 fault 3
				/*| PMFDMPB_DMP30_MASK*/		// Disable PWM3 fault 0
				/*| PMFDMPB_DMP31_MASK*/		// Disable PWM3 fault 1
				/*| PMFDMPB_DMP32_MASK*/		// Disable PWM3 fault 2
				/*| PMFDMPB_DMP33_MASK*/		// Disable PWM3 fault 3
				);

// PMF Disable Mapping C Register (PMFDMPC)
	PMFDMPC	=	(0
				/*| PMFDMPC_DMP40_MASK*/		// Disable PWM4 fault 0
				/*| PMFDMPC_DMP41_MASK*/		// Disable PWM4 fault 1
				/*| PMFDMPC_DMP42_MASK*/		// Disable PWM4 fault 2
				/*| PMFDMPC_DMP43_MASK*/		// Disable PWM4 fault 3
				/*| PMFDMPC_DMP50_MASK*/		// Disable PWM5 fault 0
				/*| PMFDMPC_DMP51_MASK*/		// Disable PWM5 fault 1
				/*| PMFDMPC_DMP52_MASK*/		// Disable PWM5 fault 2
				/*| PMFDMPC_DMP53_MASK*/		// Disable PWM5 fault 3
				);

// PMF Output Control Bit Register (PMFOUTC)
	PMFOUTC	=	(0
				/*| PMFOUTC_OUTCTL0_MASK*/		// PWM pin software control disabled
				/*| PMFOUTC_OUTCTL1_MASK*/		// PWM pin software control disabled
				/*| PMFOUTC_OUTCTL2_MASK*/		// PWM pin software control disabled
				/*| PMFOUTC_OUTCTL3_MASK*/		// PWM pin software control disabled
				/*| PMFOUTC_OUTCTL4_MASK*/		// PWM pin software control disabled
				/*| PMFOUTC_OUTCTL5_MASK*/		// PWM pin software control disabled
				);
// PMF Output Control Bit Register (PMFOUTB)
	PMFOUTB	=	(0
				/*| PMFOUTB_OUT0_MASK*/			// No direct control of pins
				/*| PMFOUTB_OUT1_MASK*/
				/*| PMFOUTB_OUT2_MASK*/
				/*| PMFOUTB_OUT3_MASK*/
				/*| PMFOUTB_OUT4_MASK*/
				/*| PMFOUTB_OUT5_MASK*/
				);

// PMF Deadtime Sample Register (PMFDTMS)
	// PMFDTMS is a read only register

// PMF Correction Control Register (PMFCCTL)
	PMFCCTL	=	(0
				/*| PMFCCTL_IPOLA_MASK*/		// PMF Value 0 register in next PWM cycle
				/*| PMFCCTL_IPOLB_MASK*/		// PMF Value 2 register in next PWM cycle
				/*| PMFCCTL_IPOLC_MASK*/		// PMF Value 4 register in next PWM cycle
				/*| PMFCCTL_ISENS0_MASK*/		// No correction
				/*| PMFCCTL_ISENS1_MASK*/
				);



// PMF Frequency Control A Register (PMFFQCA)
// Prescaler A
	PMFFQCA_PRSCA = 1;	// PWM clock frequency is f(bus)/2
// Load frequency
	PMFFQCA_LDFQA = 0;	// Reload every PWM opportunity

// PMF Counter A Register (PMFCNTA)
	// PMFCNTA is a read only register

// PMF Counter Modulo A Register (PMFMODA)
	PMFMODA = Modulus;	// See PMF.h

// PMF Deadtime A Register (PMFDTMA)
	PMFDTMA = 40;			// Insert this many f(bus) cycles for dead time
 
//  PMF Value 0 Register (PMFVAL0)
	PMFVAL0	=	Modulus/2;	// Equal positive and negative periods produce 0 volts

// PMF Enable Control A Register (PMFENCA)
	PMFENCA_LDOKA = 1;		// Load prescaler A, modulus A, and PWM0-1	
	PMFENCA_PWMENA = 1;		// Enable PWM generator A and PWM0-1

	Phase = TRUE;			// 1 = top half, 0 = bottom half
	Slope = TRUE;			// 1 = going up, 0 = going down
	Angle = 0;				// Start at zero address is sine table
	//Amp = 394;			// Maximum amplitude for test
	Amp = 0;				// Starting amplitude is zero

	PMFENCA_PWMRIEA = 1;	// Enable PWM A interrupts



// PMF Frequency Control B Register (PMFFQCB)
	PMFFQCB	=	0;
// PMF Counter B Register (PMFCNTB)
	// PMFCNTB is a read only register
// PMF Counter Modulo B Register (PMFMODB)
	PMFMODB	=	0;
// PMF Deadtime B Register (PMFDTMB)
	PMFDTMB	=	0;	
// PMF Enable Control B Register (PMFENCB)
	PMFENCB	=	0;



// PMF Frequency Control C Register (PMFFQCC)
	PMFFQCC	=	0;
// PMF Counter C Register (PMFCNTC)
	// PMFCNTC is a read only register
// PMF Counter Modulo C Register (PMFMODC)
	PMFMODC	=	0;
// PMF Deadtime C Register (PMFDTMC)
	PMFDTMC	=	0;
// PMF Enable Control C Register (PMFENCC)
	PMFENCC	=	0;

// After this instruction, write-protected registers can't be modified
	PMFCFG0_WP = 1;

/*
	EnableHBridge();	// Turn on H-bridge
*/

}


/*****************************************************************************/
void IncAmplitude(void)
{
	if (Amp < 214)
		Amp++;
}


/*****************************************************************************/
void DecAmplitude(void)
{
	if (Amp > 0)
		Amp--;
}


/*****************************************************************************/
void DisplayAmplitude(void)
{
	char* BufNum;

	BufNum = GetBuffer();		// Get a free buffer
	(void)sprintf(BufNum,"Amplitude: %d\r\n",Amp);
	SCI0_PutQueue(BufNum);
}


/*****************************************************************************/
void EnableHBridge(void)	// Shutdown signal
{
	PTP_PTP2 = 0;			// Enable H-bridge drivers	
}


/*****************************************************************************/
void DisableHBridge(void)	// Shutdown signal
{
	PTP_PTP2 = 1;			// Enable H-bridge drivers	
}
