/******************************************************************************
  PWM.c
  Driver for the Pulse Width Modulator. The PWM drives the voltage boost circuit.
******************************************************************************/

/* Include Files */
#include <mc9s12e128.h>
#include <stdio.h>
#include "PWM.h"
#include "Buffers.h"
#include "SCI0.h"



/* Global Variables */
extern word ATD_Result0;		// AC Current Sense Micro
extern word ATD_Result1;		// AC V In Voltage Sense Micro
extern word ATD_Result2;		// Input Current Sense Micro
extern word ATD_Result3;		// Input Voltage Sense Micro
extern word ATD_Result4;		// Bus Voltage Sense Micro



/******************************************************************************
   Initialize PWM Registers */
void InitPWM(void)
{
	/* PWM Polarity Register (PWMPOL) */
	/* Read: anytime. Write: anytime. */
	/* The starting polarity of each PWM channel waveform is determined by the
	   associated PPOLx bit in the PWMPOL register. If the polarity bit is 1,
	   the PWM channel output is high at the beginning of the cycle and then
	   goes low when the duty count is reached. Conversely, if the polarity bit
	   is 0 the output starts low and then goes high when the duty count is reached.

	   0 PWM channel output is low at the beginning of the period, then goes
	     high when the duty count is reached.
	   1 PWM channel output is high at the beginning of the period, then goes
	     low when the duty count is reached. **/

	PWMPOL	=	(0
				| PWMPOL_PPOL0_MASK
				| PWMPOL_PPOL1_MASK
				| PWMPOL_PPOL2_MASK
				| PWMPOL_PPOL3_MASK
				| PWMPOL_PPOL4_MASK
				| PWMPOL_PPOL5_MASK
				);



	/* PWM Clock Select Register (PWMCLK) */
	/* Read: anytime. Write: anytime. */
	/* Each PWM channel has a choice of two clocks to use as the clock source
	   for that channel. Channels 0, 1, 4, and 5 can select Clock A or Clock SA
	   as te source. Channels 2 and 3 can select Clock B or Clock SB.

 	   Pulse Width Channel 0 Clock Select
	   0 Clock A is the clock source for PWM channel 0.
	   1 Clock SA is the clock source for PWM channel 0.

	   Pulse Width Channel 2 Clock Select
	   0 Clock B is the clock source for PWM channel 2.
	   1 Clock SB is the clock source for PWM channel 2. */

	PWMCLK	=	(0
				/*| PWMCLK_PCLK0_MASK*/
				/*| PWMCLK_PCLK1_MASK*/
				/*| PWMCLK_PCLK2_MASK*/
				/*| PWMCLK_PCLK3_MASK*/
				/*| PWMCLK_PCLK4_MASK*/
				/*| PWMCLK_PCLK5_MASK*/
				);



	/* PWM Prescale Clock Select Register (PWMPRCLK) */
	/* This register selects the prescale clock source for clocks A and B
	   independently. */
	/*----+-------+-------+-------+-------+-------+-------+-------+-------*/
	/*    |   7   |   6   |   5   |   4   |   3   |   2   |   1   |   0   */
	/*----+-------+-------+-------+-------+-------+-------+-------+-------*/
	/* R  |   0   |       |       |       |   0   |       |       |       */
	/*----+-------| PCKB2 | PCKB1 | PCKB0 |-------| PCKA2 | PCKA1 | PCKA0 */
	/* W  |\\\\\\\|       |       |       |\\\\\\\|       |       |       */
	/*----+-------+-------+-------+-------+-------+-------+-------+-------*/
	/* Rst	  0		  0		  0		  0		  0		  0		  0		  0   */

	PWMPRCLK	=	(0
					| PWMPRCLK_PCKA0_MASK			// Clock A pre-scale = bus clock/2
					/*| PWMPRCLK_PCKA1_MASK*/
					/*| PWMPRCLK_PCKA2_MASK*/

					| PWMPRCLK_PCKB0_MASK			// Clock B pre-scale = bus clock/2
					/*| PWMPRCLK_PCKB1_MASK*/
					/*| PWMPRCLK_PCKB2_MASK*/
					);



	/* PWM Center Align Enable Register (PWMCAE) */
	/* Read: anytime. Write: anytime. */
	/* The PWMCAE register contains six control bits for the selection of
	   center aligned outputs or left aligned outputs for each PWM channel. If
	   the CAEx bit is set to a 1, the corresponding PWM output will be center
	   aligned. If the CAEx bit is cleared, the corresponding PWM output will
	   be left aligned.

	   0 Channel x operates in left aligned output mode.
	   1 Channel x operates in center aligned output mode. */

	PWMCAE	=	(0
				/*| PWMCAE_CAE0_MASK*/
				/*| PWMCAE_CAE1_MASK*/
				/*| PWMCAE_CAE2_MASK*/
				/*| PWMCAE_CAE3_MASK*/
				/*| PWMCAE_CAE4_MASK*/
				/*| PWMCAE_CAE5_MASK*/
				);



	/* PWM Control Register (PWMCTL) */
	/* Read: anytime. Write: anytime. */
	/* The PWMCTL register provides for various control of the PWM module. */
	/*----+-------+-------+-------+-------+-------+-------+-------+-------*/
	/*    |   7   |   6   |   5   |   4   |   3   |   2   |   1   |   0   */
	/*----+-------+-------+-------+-------+-------+-------+-------+-------*/
	/* R  |   0   |       |       |       |       |       |   0   |   0   */
	/*----+-------| CON45 | CON23 | CON01 | PSWAI | PFRZ  +-------+-------*/
	/* W  |\\\\\\\|       |       |       |       |       |\\\\\\\|\\\\\\\*/
	/*----+-------+-------+-------+-------+-------+-------+-------+-------*/
	/* Rst	  0		  0		  0		  0		  0		  0		  0		  0   */

	PWMCTL	=	(0
				/*| PWMCTL_PFRZ_MASK*/		// 1: stop in freeze mode
				/*| PWMCTL_PSWAI_MASK*/		// 1: stop in wait mode
				/*| PWMCTL_CON01_MASK*/		// 1: concatinate channels 0 & 1
				/*| PWMCTL_CON23_MASK*/		// 1: concatinate channels 2 & 3
				/*| PWMCTL_CON45_MASK*/		// 1: concatinate channels 4 & 5
				);



	/* PWM Scale A Register (PWMSCLA) */
	/* Read: anytime. Write: anytime (causes the scale counter to load the PWMSCLA value). */
	/* PWMSCLA is the programmable scale value used in scaling clock A to
	   generate clock SA. Clock SA is generated by taking clock A, dividing it
	   by the value in the PWMSCLA register and dividing that by two.
	   Clock SA = Clock A / (2 * PWMSCLA)
	   NOTE:
	   When PWMSCLA = 0x0000, PWMSCLA value is considered a full scale value of
	   256. Clock A is thus divided by 512.

	   Any value written to this register will cause the scale counter to load
	   the new scale value (PWMSCLA). */

	PWMSCLA	=	(0
				/*| PWMSCLA_BIT0_MASK*/
				/*| PWMSCLA_BIT1_MASK*/
				/*| PWMSCLA_BIT2_MASK*/
				/*| PWMSCLA_BIT3_MASK*/
				/*| PWMSCLA_BIT4_MASK*/
				/*| PWMSCLA_BIT5_MASK*/
				/*| PWMSCLA_BIT6_MASK*/
				/*| PWMSCLA_BIT7_MASK*/
				);



	/* PWM Scale B Register (PWMSCLB) */
	/* Read: anytime. Write: anytime (causes the scale counter to load the PWMSCLB value). */
	/* PWMSCLB is the programmable scale value used in scaling clock B to
	   generate clock SB. Clock SB is generated by taking clock B, dividing it
	   by the value in the PWMSCLB register and dividing that by two.
	   Clock SB = Clock B / (2 * PWMSCLB)
	   NOTE:
	   When PWMSCLB = 0x0000, PWMSCLB value is considered a full scale value of
	   256. Clock B is thus divided by 512.

	   Any value written to this register will cause the scale counter to load
	   the new scale value (PWMSCLB). */

	PWMSCLB	=	(0
				/*| PWMSCLB_BIT0_MASK*/
				/*| PWMSCLB_BIT1_MASK*/
				/*| PWMSCLB_BIT2_MASK*/
				/*| PWMSCLB_BIT3_MASK*/
				/*| PWMSCLB_BIT4_MASK*/
				/*| PWMSCLB_BIT5_MASK*/
				/*| PWMSCLB_BIT6_MASK*/
				/*| PWMSCLB_BIT7_MASK*/
				);



	/* PWM Channel Period Registers (PWMPERx) */
	/* Read: anytime. Write: anytime. */
	/* There is a dedicated period register for each channel. The value in this
	   register determines the period of the associated PWM channel.

	   The period registers for each channel are double buffered so that if they
	   change while the channel is enabled, the change will NOT take effect until
	   one of the following occurs:
	   1 The effective period ends
	   2 The counter is written (counter resets to 0x0000)
	   3 The channel is disabled

	   In this way, the output of the PWM will always be either the old waveform
	   or the new waveform, not some variation in between. If the channel is not
	   enabled, then writes to the period register will go directly to the
	   latches as well as the buffer.
	   NOTE:
	   Reads of this register return the most recent value written. Reads do not
	   necessarily return the value of the currently active period due to the
	   double buffering scheme. */

	PWMPER0	=	0xFF;	// Full scale of 255 clocks
	PWMPER1	=	0xFF;
	PWMPER2	=	0xFF;
	PWMPER3	=	0xFF;
	PWMPER4	=	0xFF;
	PWMPER5	=	0xFF;



	/* PWM Channel Duty Registers (PWMDTYx) */
	/* Read: anytime. Write: anytime. */
	/* The value in this register determines the duty of the associated PWM channel. */
	PWMDTY0	=	0x00;	// PWM stopped
	PWMDTY1	=	0x00;
	PWMDTY2	=	0x00;
	PWMDTY3	=	0x00;
	PWMDTY4	=	0x00;
	PWMDTY5	=	0x00;



	/* PWM Shutdown Register (PWMSDN) */
	/* Read: anytime. Write: anytime. */
	/* The PWMSDN register provides for the shutdown functionality of the PWM module in the emergency cases. */
	PWMSDN	=	(0
				/*| PWMSDN_PWM5ENA_MASK*/	// 0 PWM emergency feature disabled. 1 PWM emergency feature is enabled.
				/*| PWMSDN_PWM5INL_MASK*/	// 0 Active level is low. 1 Active level is high.
				/*| PWMSDN_PWM5IN_MASK*/
				/*| PWMSDN_PWMLVL_MASK*/	// 0 PWM outputs are forced to 0. 1 PWM outputs are forced to 1.
				/*| PWMSDN_PWMRSTRT_MASK*/
				/*| PWMSDN_PWMIE_MASK*/		// 0 PWM interrupt is disabled. 1 PWM interrupt is enabled.
				/*| PWMSDN_PWMIF_MASK*/		// 0 No change on PWM5IN input. 1 Change on PWM5IN input
				);



	/* PWM Enable Register (PWME) */
	/* Read: anytime. Write: anytime. */
	/* 0 Pulse width channel is disabled. 1 Pulse width channel is enabled. The pulse modulated signal becomes
	   available at PWM,output when its clock source begins its next cycle. */
	PWME	=	(0
				/*| PWME_PWME0_MASK*/	// 0 Pulse width channel 0 is disabled. 1 Pulse width channel 0 is enabled.
				/*| PWME_PWME1_MASK*/	// Flip-Flop A Reset output
				/*| PWME_PWME2_MASK*/	// 0 Pulse width channel 2 is disabled. 1 Pulse width channel 2 is enabled.
				/*| PWME_PWME3_MASK*/	// Flip-Flop B Reset output
				/*| PWME_PWME4_MASK*/	// Not available on 80-pin part
				/*| PWME_PWME5_MASK*/	// Not available on 80-pin part
				);
}

/******************************************************************************
   Start PWM for voltage boost */
void StartBoostPWM(void)
{
	PWMDTY0 = 1;		// Start with low duty cycle
	PWME_PWME0 = 1;		// Enable PWM channel 0 - Boost1 and Boost2 signals
	PTP_PTP3 = 1;		// Enable boost MOSFET driver - Enable signal
}

/******************************************************************************
   Stop PWM for voltage boost */
void StopBoostPWM(void)
{
	PWMDTY0 = 0;		// No duty cycle
	PWME_PWME0 = 0;		// Disable PWM channel 0 - Boost1 and Boost2 signals
	PTP_PTP3 = 0;		// Disable boost MOSFET driver - Enable signal
}

/******************************************************************************
   Increment PWM0 */
void IncrementPWM0(void)
{
	if (PWMDTY0 < 254)		// A value of 255 is always on so don't use
		PWMDTY0++;
}

/******************************************************************************
   Decrement PWM0 */
void DecrementPWM0(void)
{
	if (PWMDTY0 > 1)		// A value of 0 is always off
		PWMDTY0--;
}

/******************************************************************************
   Control loop for boost PWM.
   Call this function in main loop. The PWM is enabled by Thread 1 State 4
   of App.c */
void MonitorBoostPWM(void)
{
	if (PWME_PWME0)		// Only if PWM has been enabled
	{
		if (ATD_Result4 < 972)			// 380 Volts
			IncrementPWM0();
		else if (ATD_Result4 > 1000)	// 391 Volts
			DecrementPWM0();
	}
}

/******************************************************************************
   Display PWM0 */
void DisplayPWM0(void)
{
	char* BufNum;

	BufNum = GetBuffer();		// Get a free buffer
	(void)sprintf(BufNum,"PWM0: %02X\r\n",PWMDTY0);
	SCI0_PutQueue(BufNum);
}


