/******************************************************************************
   SCI1.c
   Interrupt driven SCI routines.
******************************************************************************/

// Include Files
#include <MC9S12E128.h>
#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "App.h"
#include "Buffers.h"
#include "ATD.h"
#include "CPU.h"
#include "DAC.h"
#include "IO.h"
#include "KBD.h"
#include "LCD.h"
#include "PMF.h"
#include "PWM.h"
#include "SCI0.h"
#include "SCI1.h"
#include "SPI.h"
#include "Timer.h"
#include "Datatypes.h"


// Global Variables
char *TxMsg1;
word MsgOutPtr1;
rx_q_struct rxbuf1;		// Receive buffer and pointers
tx_q_struct txqueue1;	// Transmit buffer queue and pointers

char RcvBuf1[RcvBufSize1];
word RcvBufPtr1;

static tPointerFcn SCI1_RxFuncPtr = SCI1_Rx1;	// Start in State 1
byte MsgInBuf = FALSE;	// Message in buffer flag

extern char WaitFlags;


/******************************************************************************
  This is the SCI1 Interrupt Handler. */
#pragma CODE_SEG __NEAR_SEG NON_BANKED
interrupt void SCI1_Int(void)
{
	char data;		// Temporary data storage

	if(SCI1SR1_RDRF)	// If received data flag is on
	{
		data = SCI1DRL;		// transfer SCI data reg to temporary storage
		if (data == DC3)	// If data is XOFF
			SCI1CR2_SCTIE = 0;	// suppress transmitter interrupts
		else if (data == DC1)	// If data is XON
		{
			/*
			SCI1CR2_SCTIE = 1;	// allow transmitter interrupts
			SCI1CR2_TCIE = 1;	// and transmission complete interrupts
			*/
			SCI1CR2 |= SCI1CR2_SCTIE_MASK + SCI1CR2_TCIE_MASK;
		}
		else
		{
			rxbuf1.q[rxbuf1.in++] = data;	// Enter character and increment pointer
			if (rxbuf1.in > (rxqsize1-1))	// If exceeded buffer length
				rxbuf1.in = 0;	// set receive buffer in pointer to start of buffer
		}

	}
	else if (!SCI1CR2_SCTIE)	// If transmitter interrupts are already disabled
	{
		SCI1CR2_TCIE = 0;	// suppress transmission complete interrupts
		SCI1CR2_TE = 0;		// and disable SCI transmitter
		WaitFlags |= 0x04;
	}
	else if (SCI1SR1_TDRE)	// If transmitter data register is empty
	{
		data = TxMsg1[MsgOutPtr1++];	// get a byte from buffer and increment pointer
		if (data == NUL)	// If data is NUL we are done with this message so
			SCI1CR2_SCTIE = 0;	// suppress transmitter interrupts
		else
			SCI1DRL = data;		// Otherwise send the data byte
	}
}
#pragma CODE_SEG DEFAULT


/******************************************************************************
  Initialize SCI1 */
void InitSCI1(void)
{
	SCI1_SetBaud();			// Set SCI0 baud rate
	SCI1_StartRx();			// Start SCI0 receiver
	SCI1_TxQueueInit();		// Initialize transmit queue pointers
}

/******************************************************************************
  This routine sets SCI1 baud rate. */
void SCI1_SetBaud(void)
{
	SCI1BD  = SCI1_BAUD_DIV;	// Set baud rate register
	SCI1CR1 = 0;
}

/******************************************************************************
  This routine initializes the receive buffer in and out pointers. */
void SCI1_RxQueueInit(void)
{
	rxbuf1.in = 0;
	rxbuf1.out = 0;
}

/******************************************************************************
  This routine starts the SCI1 receiver. */
void SCI1_StartRx(void)
{
	SCI1_RxQueueInit();
/*
	SCI1CR2_RIE = 1;	// Enable receiver interrupts
	SCI1CR2_RE = 1;		// Enable receiver
*/
	SCI1CR2 |= SCI1CR2_RIE_MASK + SCI1CR2_RE_MASK;
}

/******************************************************************************
  This routine stops the SCI1 receiver to conserve power */
void SCI1_StopRx(void)
{
/*
	SCI1CR2_RIE = 0;	// Disable receiver interrupts
	SCI1CR2_RE = 0;		// Disable receiver
*/
	SCI1CR2 &= ~(SCI1CR2_RIE_MASK + SCI1CR2_RE_MASK);
}

/******************************************************************************
  This routine initializes the transmit buffer queue in and out pointers */
void SCI1_TxQueueInit(void)
{
	txqueue1.in = 0;
	txqueue1.out = 0;
}

/******************************************************************************
  This routine makes an entry into the transmit queue of pointers and updates
  the in pointer to the next available location. */
void SCI1_PutQueue(char *msg)
{
	txqueue1.q[txqueue1.in++] = msg;
	if (txqueue1.in > (txqsize1-1))
		txqueue1.in = 0;
}

/******************************************************************************
  This routine starts the transmitter */
void SCI1_StartTx(void)
{
/*
	SCI1CR2_SCTIE = 1;	// Enable transmitter interrupts
	SCI1CR2_TCIE = 1;	// Enable transmission complete interrupts
	SCI1CR2_TE = 1;		// Enable the SCI1 transmitter
*/
	SCI1CR2 |= SCI1CR2_SCTIE_MASK + SCI1CR2_TCIE_MASK + SCI1CR2_TE_MASK;
}

/******************************************************************************
  This routine retrieves a byte from the receive buffer if present. Returns 0
  if no data available. Returns 1 if a data byte has been transferred
  successfully. The pointer to the data byte is passed to the routine. */
byte SCI1_GetChar(char *rxbyte)
{
	if (rxbuf1.in != rxbuf1.out)
	{
		*rxbyte = rxbuf1.q[rxbuf1.out++];  // Read character and update pointer
		if (rxbuf1.out > (rxqsize1-1))  // If exceeded buffer length
			rxbuf1.out = 0;  // set receive buffer out pointer to start of buffer
		return 1;
	}
	else
	{
		return 0;
	}
}

/******************************************************************************
  This routine enters a pointer to a text message and starts the SCI1
  transmitter. */
void SCI1_SendMsg(char *s)
{
	TxMsg1 = s;			// Pointer to the message for the SCI0 interrupt handler
	MsgOutPtr1 = 0;		// Point to start of text message
	SCI1_StartTx();		// Turn on SCI1 transmitter
}

/******************************************************************************
  This routine checks the transmit buffer queue and if it is not empty,
  it extracts the pointer to the message and sets up the SCI1 transmit
  interrupt handler. */

/* Call this function in main loop */
void SCI1_CheckQueue(void)
{
	if (!SCI1CR2_TE)	// Don't try to start transmitter if already transmitting
		if (txqueue1.in != txqueue1.out)	// Look for a pointer in the queue and
		{
			SCI1_SendMsg(txqueue1.q[txqueue1.out++]);	// hand it off. Update pointer
			if (txqueue1.out > (txqsize1-1))	// If past the end of array
				txqueue1.out = 0;	// set pointer to start of array
			WaitFlags &= ~0x04;
		}
}



/******************************************************************************
  SCI1_Rx State Machine */

/* Call this function in main loop */
void SCI1_Rx(void)
{
	SCI1_RxFuncPtr();	// Use the function pointer to go to a state
}

/******************************************************************************
  SCI1 State Rx1: Accumulate received characters in a buffer until a newline
  character is received then insert a NUL character into the buffer and go to
  State 2. */
void SCI1_Rx1(void)
{
	char rxchar;

	if (SCI1_GetChar(&rxchar))
	{
		switch (rxchar)
		 {
			 case '\n':
				RcvBuf1[RcvBufPtr1++] = rxchar;
				RcvBuf1[RcvBufPtr1++] = 0;
				RcvBufPtr1 = 0;
				SCI1_RxFuncPtr = SCI1_Rx2;
				break;

			default:
				if (RcvBufPtr1 < (RcvBufSize1-2))
					RcvBuf1[RcvBufPtr1++] = rxchar;
				break;
		}
	}
}


/******************************************************************************
  SCI1 State Rx2: Wait for someone to release the buffer by setting MsgInBuf
  to FALSE. Then reset the receive buffer pointer and go to State 1. */
void SCI1_Rx2(void)
{
	switch (RcvBuf1[0])
	{
		case 'a':		/* Change to suit application */
			SCI1_PutQueue("Command a received\r\n");
			Toggle_LED();
			break;
		case 'b':
			SCI1_PutQueue("ACK\r\n");	// 'ACK'
			break;
		case 'c':
			break;
		case 'd':
			break;
		case 'e':
			Toggle_LED();
			SCI1_PutQueue("ACK\r\n");	// 'ACK'
			break;
		case 'f':
			break;
		case 'g':
			break;
		case 'h':
			break;


		default:
			SCI1_PutQueue("Unknown command\r\n");
			break;
	}
	SCI1_RxFuncPtr = SCI1_Rx1;
}
